class simple_charpy:
    def __init__(self, corrected_energy: float, thickness: float, width: float):
        """Class for calculating the non-instrumented Charpy impact strength. 

        In general by using this class you can calculate both notched and unnotched samples. For notched you need to put
        remained width on notch place.

        For now it is only for non-instrumented tests

        Parameters
        ----------
        corrected_energy : float
            it it the energy absorbed by breaking the test specimen
        thickness : float
            thickness of the test specimen
        width : float
            width of the test specimen

        Examples
        --------
        FIXME: Add docs.

        """
        
        self.impact_strength=(corrected_energy)/(thickness*width)



import glob,os
import re
class files:
    def __init(self):
        pass
        

    def find(self,path, extension:str,methods=None, mods=None):
        """This class can manage the files as well as it's parameters

        This class helps managing he files, find their names/paths, extract the method of manufacturing or other
        parameters

        TODO: add automatically finding the method placement in names, by putting them in the method definition
        
        Parameters
        ----------
        path : str
            This variable is used to put path to your files
        extension : str
            Extention is the extention of your files. If your file is .csv put csv in there, be aware and do not put the dot before your extention
        methods : str
            Descripe the technique of manufacturing there, or find it using find_method in that class

        Examples
        --------
        FIXME: Add docs.

        """

        methods = [method.upper() for method in methods]
        if extension.startswith("."):
            extension=extension[1:]
        
        files=glob.glob(os.path.join(path, f"*.{extension}"))
        # breakpoint()
        try:
            self.files=[file for file in files if any(method in file for method in methods)] 
            splitted = [os.path.split(file)[1] for file in self.files if any(method in file for method in methods)] #split to extract names
            self.paths = [file for file in self.files if any(method in file for method in methods)]
        except TypeError:
            self.files=[file for file in self.files if any(method in file for method in methods)] 
            splitted = [os.path.split(file)[1] for file in self.files if any(method in file for method in methods)]
            self.paths = [file for file in self.files]
        self.names = [name.split(".")[0] for name  in splitted]
        numbers = [name.split("_")[1] for name  in splitted]
        self.numbers=[os.path.splitext(number)[0] for number in numbers]
        self.comments= [self.find_comments(name)[0] for name in self.names]
        self.clean_names=[self.find_comments(name)[1] for name in self.names]
        if methods is not None:
            self.methods = [self.find_method(name, methods) for name in self.names ] #thats for
        if mods is not None:
            self.modifications = [self.find_modification(name, mods) for name in self.names]
        self.composition=[self.find_composition(name.split("_")[0]) for name in self.names]
    def find_method(self,name:str,
                    methods:list,
                    delimiter:str='-', ):
        """Find the technique how the material was created

        Find the method how the material was created, what methods were used to modify it, etc. To do so it is using
        first letters of filename, so for extruded parts you can use EXT, for annealed extruded parts you can use aEXT
        etc.

        Parameters
        ----------
        path : str
            path for your file
        delimiter : str
            what sign you wanna use to finish your method string. By default it is -

        Examples
        --------
        FIXME: Add docs.

        """
        name=name.split(delimiter)[0]
        for method in methods:
            if method in name:
                return method

    
    def find_modification(self, name, mods:list, place=0):
        if not  isinstance(mods, list):
            raise ValueError("Put list of modifications!")
        for mod in mods:
            letter = mod[0]
            if name[place]==letter:
                return mod
            else:
                continue
        return None
    
    def find_composition(name,delimiter: str='-', percent_sign="p"):
        """
        TODODO baby shark
        method to obtain material ingridiens from name, as I usually name files extracted from machine with code allowing me to get that information from filename
        """
        # breakpoint()
        splitted=name.split(delimiter)
        comp={}
        for name in splitted[1:]:
            comp.update(_extract_info(name, percent_sign))
        return comp




    def find_comments(self,s):

        pattern = r'\[.*?\]'  # a regular expression pattern to match text between square brackets
        matches = re.findall(pattern, s)  # find all matches of the pattern in the string
        comments = ''.join(matches)  # concatenate all the matches into a single string
        cleaned_text = re.sub(pattern, '', s)  # remove all the matches from the original string
        comments = re.sub(r'[\[\]]', '', comments)  # remove square brackets from the removed text
        return comments, cleaned_text

def _extract_info(s, delimiter="p"):
    pattern = rf'(\d+){delimiter}(.+)'
    match = re.match(pattern, s)
    result = {}
    if match:
        result[match.group(2)] = [int(match.group(1))]
    # else:
    #     return None
    return result    



import numpy as np
import matplotlib.pyplot as plt
from typing import Union
import importlib



class _engineering_values:
        def __init__(self,
                     width,
                     thickness,
                     name=None,
                     elongation_array: Union[list, np.array] =None,
                     force_array: Union[list, np.array] =None,
                     force_units="N",
                     length_units="mm",
                   norm="ISO527"
                     ):
            global properties
            properties = importlib.import_module(f".{norm}", package=__package__)
            """initializer of engineering values class

            This class is used to menage the properties of engineering values

            Parameters
            ----------
            name : str
                name for your sample

            Examples
            --------
            FIXME: Add docs.

            """
            
            self.name=name
            self.force_units = force_units
            self.length_units=length_units
            self.thickness=thickness,
            self.width=width
            self.stress, self.strain=None,None
        def _calculate(self,
                      thickness:float,
                      width:float,
                      elongation_array,
                      force_array,
                      ):
            """Calculates the engineering stress and strain

            This method is used to calculate engineering stress and strain from height

            Parameters
            ----------
            thickness : float
                smaller initial dimension of the rectangular cross-section in the central part of test specimen
            width : float
                larger initial dimension of the rectangular cross-section in the central part of the test specimen
            elongation_array : list
                list of the sample elongation
            force_array : list
                list of the forces from the machinge
            force_units : str
                Units used as force units. Newtons [N] by default. Use shorten strings, as it is used in output
            length : str
                Units used for lenght, mm by default. Use short strings, as they are used in output. If you wanna use percent, just use % sign or percent
            """
            

            if self.length_units=="%" or self.length_units=="percent":
                self.percent_strain=True
            else:
                self.percent_strain=False


            if  self.stress is  None and self.strain is None:
                initial_area=thickness*width
                self.stress=[force/initial_area for force in force_array]
                if self.percent_strain:
                    self.strain=[strain for strain in elongation_array]
                else:
                    self.strain=[strain*0.01 for strain in elongation_array]

            self.strength=self._calculate_strength(self.stress,
                                                  self.strain)
            
            self.at_break=self._calculate_at_break(self.strain,
                                                  self.stress)
            
            self.yield_strength=self._calculate_yield_strength(self.stress,
                                                              self.strain)
            self.tensile_modulus=self._calculate_tensile_modulus()

        class _calculate_strength:
            def __init__(self,strain: np.array,stress:np.array):
                """Strenght is according to ISO-527-1 first maximum local value

                Parameters
                ----------
                strain : np.array
                    strain array
                stress : np.array
                    stress array

                Examples
                --------
                FIXME: Add docs.

                """
                
                strength = properties.strenght(strain,stress)
                self.value=strength.value
                self.strain=strength.strain

        class _calculate_yield_strength:
            def __init__(self, stress: np.array, strain: np.array):
                """
                Yield strenght  is according to ISO-527-1 strain increase without stress increase. 

                Parameters
                ----------
                stress : np.array
                    stress numpy array
                strain : np.array
                    strain numpy array

                Examples
                --------
                FIXME: Add docs.

                """
                yield_strenght=properties.yield_strenght(stress, strain)
                self.stress, self.strain=yield_strenght.value,yield_strenght.strain
        class _calculate_at_break:
            """
            This class calculates values at the brain according to ISO 527-1
            """
            def __init__(self,stress, strain):
                at_break=properties.at_break(stress, strain)
                self.stress=at_break.stress
                self.strain=at_break.strain


        def _calculate_tensile_modulus(self,
                                      plot=False,
                                      r2=True,
                                      output=False,
                                      lower_limit=0.05,
                                      upper_limit=0.25
                                      ):
            """            Tensile, or Young's modulus is the slope of strain/stress curve, between strains equals to 0.05 and 0.25 percent according to DIN ISO 527-1

            Parameters
            ----------
            plot : bool
                Put True for use pyplot on the tensile modulus part
            r2 : Put coeffitient 	of determination into pyplot label
                Put coeffitient of determination into pyplot label
            output : bool
                Print the output of this method
            lower_limit : float
                lower limit of the measurement boundary
            upper_limit : float
                upper limit of the measurement boundary

            Examples
            --------
            FIXME: Add docs.

            """

            E=properties.tensile_modulus(self.stress,
                                         self.strain,
                                         percent_strain=self.percent_strain
                                         )
            if plot:
                label=rf"Young's modulus {int(E.tensile_modulus)} $\left[\frac{{{self.force_units}}}{{{self.length}^2}}\right]$"
                if r2:
                    label+="\n"+rf"$R^{{{2}}}={E.r2}$"
                    plt.plot(E.module_strain,
                             E.module_stress,
                             label=label
                             )
            if output:
                print(f"Tensile modulus is equal to {int(E.tensile_modulus)} [{self.force_units}/{self.length}^2]")
            return   E.tensile_modulus

                
        def set(self,
                engineering_stress: Union[list, np.array] =None,
                engineering_strain:Union[list, np.array] =None,
                # TODO: gives TypeError: only size-1 arrays can be converted to Python scalars while using numpy array
                # TODO: gives KeyKerrr None
                # TODO: in general this method does not work yet!
                ):
            """Method to set engineering stress and engineering strain

            Parameters
            ----------
            engineering_stress : list
                array of engineering stress
            engineering_strain : list
                array of engineering strain

            Examples
            --------
            FIXME: Add docs.

            """
            
            self.stress, self.strain=engineering_stress, engineering_strain
            self._calculate(thickness=self.thickness,
                           width=self.width,
                           elongation_array=None,
                           force_array=None)
            

        def plot(self, show=False):
            """Method for plotting the results

            This method can be used to plot your engineering stress-strain curve. If you wanna show it instantly use
            parameter show as True

            Parameters
            ----------
            show : bool
                It it equal to matplotlib.pyplot function show

            Examples
            --------
            FIXME: Add docs.

            """
            plt.plot(self.strain,self.stress, label=self.name)
            plt.title(self.name)
            plt.ylabel(rf"Stress $\left[\frac{{{self.force_units}}}{{{self.length_units}^2}}\right]$")
            plt.xlabel(f"Strain [{self.length_units}]")
            plt.legend()
            if show:
                plt.show()

class _real_values(_engineering_values):

        def __init__(self,
                     name,
                     thickness,
                     width,
                     stress,
                     strain,
                     force_units,
                     length_units,
                     ):
            self.force_units=force_units
            self.length_units=length_units
            if length_units=="%" or length_units=="percent":
                self.percent_strain=True
            else:
                self.percent_strain=False

    
            _engineering_values.__init__(self,
                                        name=name,
                                        thickness=thickness,
                                        width=width
                                        )
            self.name=name + " [real]"
            width=self.width
            thickness=self.thickness
            super().__init__(self,
                             name=self.name,
                             thickness=self.thickness
                             )
            self._calculate(stress, strain)

        def _calculate(self, stress, strain):
            """
                Calculates the true stress and strain, from engineering values.
                Read more there:
                        https://courses.ansys.com/index.php/courses/topics-in-metal-plasticity/lessons/how-to-define-a-multilinear-hardening-plasticity-model-lesson-1/
            
                Parameters:
                        strain (array-like): An array of strain values.
                        stress (array-like): An array of corresponding stress values.
                        n: An count of chunks to divide stress/strain array
            
                Returns:
                    A tuple (start_strain, end_strain) representing the proportional range of the stress-strain curve.
            """
            self.stress=[stress_val*(1+strain_val) for stress_val, strain_val in zip(stress, strain)]
            self.strain=[np.log(1+strain_val) for strain_val in strain]

            self.strength=self._calculate_strength(self.stress,
                                                  self.strain)
            
            self.at_break=self._calculate_at_break(self.strain,
                                                  self.stress)
            
            self.yield_strength=self._calculate_yield_strength(self.stress,
                                                              self.strain)
            self.tensile_modulus=self._calculate_tensile_modulus()            






class _tensile:
    
    def __init__(self,
                 name: str,
                 thickness: Union[float, int],
                 width: Union[float, int],
                 elongation_array: Union[list, np.array] = None,
                 force_array: Union[list, np.array] = None,
                 stress_array: Union[list, np.array] = None,
                 strain_array: Union[list, np.array] = None,
                 force_units: str = "N",
                 length_units: str = "mm"
                 ):
        self.name, self.thickness, self.width=name, thickness, width
        self.engineering_values = _engineering_values(
            width=width,
            thickness=thickness,
            name=name,
            elongation_array=elongation_array,
            force_array=force_array,
            force_units=force_units,
            length_units=length_units
        )
        self.force_units, self.length_units = force_units, length_units
        self.stress_units = f"{force_units}/{length_units}^2"
        self.strain_units = f"{length_units}/{length_units}"
        if stress_array is None and strain_array is None:
            if elongation_array is None and force_array is None:
                raise ValueError("None of elongation/force or stress/strain arrays are defined!")
            elif thickness is None:
                raise ValueError("Thickness is not defined!")
            elif width is None:
                raise ValueError("Width is not defined!")
            else:
                self.elongation_array = elongation_array
                self.force_array = force_array
                self.engineering_values._calculate(thickness, width, elongation_array, force_array)
        else:
            self.engineering_values.set(stress_array, strain_array)

    def convert2real(self):
        self.real_values = _real_values(self.name,
                                        self.thickness,
                                        self.width,
                                        self.engineering_values.stress,
                                        self.engineering_values.strain,
                                        self.force_units,
                                        self.length_units)





import numpy as np
from .... import _misc


"""
According to ISO 527-1: https://cdn.standards.iteh.ai/samples/75824/61c480ef4bf0494aa6966bd4c2244c2e/ISO-527-1-2019.pdf
"""
class tensile_modulus:
    def __init__(self,
                 stress,
                 strain,
                 percent_strain=False,
                 lower_limit=0.05,
                 upper_limit=0.25
                 ):
        """
        Tensile, or Young's modulus is the slope of strain/stress curve, between strains equals to 0.05 and 0.25 percent
        """
        stress=np.array(stress)
        strain=np.array(strain)
        if not percent_strain:
            strain=strain*100
        module_strain=strain[(strain>lower_limit) & (strain<upper_limit)]
        low_idx = np.where(strain==min(module_strain))[0][0]
        upp_idx = np.where(strain==max(module_strain))[0][0]
        module_stress = stress[low_idx:upp_idx+1]
        if not percent_strain:
            module_strain=module_strain/100
        array = np.vstack([module_strain, np.ones(len(module_strain))]).T

        E, c = np.linalg.lstsq(array, module_stress, rcond=-1)[0]
        r2 = 1 - c / (module_stress.size * module_stress.var())
        self.r2 = round(r2,4)
        self.tensile_modulus=_misc._round_sig(E, sig=3)
        self.module_strain, self.module_stress =module_strain, module_stress


class at_break:
    def __init__(self, stress, strain):
        self.stress=_misc._round_sig(stress[len(stress)-1], sig=3)
        self.strain=_misc._round_sig(strain[len(strain)-1])
        
class strenght:
    def __init__(self, stress, strain):
        self.value=_misc._round_sig(self.find_local_maximum(stress), sig=3)
        self.strain=_misc._round_sig(strain[self.idx])
    def find_local_maximum(self, stress):
        for i in range(1, len(stress)-1):
            if stress[i-1] < stress[i] > stress[i+1] and stress[i]>1:
                self.idx=i
                return  stress[i]

        print("Strength: Local maximum not found, gives max value")
        self.idx=np.where(stress==np.max(stress))[0][0]
        val = np.max(stress)
        return val
                

class yield_strenght:
    def __init__(self,stress, strain):
        self.value=_misc._round_sig(stress[self.find_idx(stress,strain)], sig=3)
        self.strain=_misc._round_sig(strain[self.find_idx(stress,strain)])
    def find_idx(self, stress,strain):
        for i in range(1, len(strain)):
            if strain[i] > strain[i-1] and stress[i] <= stress[i-1]:
                if stress[i]>1:
                    return i
                else:
                    continue
                    # exit()
        return len(strain)-1

        

        
            
